@@ -4,8 +4,7 @@ Main Wikipedia client with async support, rate limiting, caching, and retry logi
 
 import asyncio
 import logging
-from typing import List, Optional, Dict, Any, Union
-import time
+from typing import List, Optional, Dict, Any
 from decimal import Decimal
 
 import aiohttp
@@ -17,9 +16,14 @@ from tenacity import (
     retry_if_exception_type,
     before_sleep_log,
 )
+import traceback
 
-from .config import ClientConfig
-from .models import (
+from wikipedia_async.helpers.section_helpers import SectionHelper
+
+from wikipedia_async.config import ClientConfig
+from wikipedia_async.models.wiki_client_model import (
+    BatchHTMLResult,
+    HTMLResult,
     SearchResult,
     WikiPage,
     PageSummary,
@@ -28,9 +32,9 @@ from .models import (
     GeoSearchResult,
     BatchResult,
     RandomPageResult,
-    SuggestionResult,
+    SearchResults,
 )
-from .exceptions import (
+from wikipedia_async.exceptions import (
     WikipediaException,
     TimeoutError,
     NetworkError,
@@ -41,10 +45,9 @@ from .exceptions import (
     APIError,
     ValidationError,
 )
-from .cache import AsyncTTLCache, BaseCache, cache_key
-
-logger = logging.getLogger(__name__)
-
+from wikipedia_async.cache import AsyncTTLCache, BaseCache, cache_key
+from urllib.parse import unquote_plus
+from wikipedia_async.helpers.logger_helpers import logger
 
 class WikipediaClient:
     """
@@ -73,7 +76,7 @@ class WikipediaClient:
             session: Optional aiohttp session (creates new if None)
         """
         self.config = config or ClientConfig()
-        self._session = session
+        self._session: aiohttp.ClientSession = session  # type: ignore
         self._owned_session = session is None
 
         # Rate limiter
@@ -120,7 +123,7 @@ class WikipediaClient:
         """Close the client and cleanup resources."""
         if self._session and self._owned_session:
             await self._session.close()
-            self._session = None
+            self._session = None  # type: ignore
 
     @retry(
         stop=stop_after_attempt(3),
@@ -151,12 +154,11 @@ class WikipediaClient:
 
         # Add standard parameters
         params.update({"action": "query", "format": "json", "formatversion": "2"})
-
+        lang = params.pop("lang", self.config.language)
+        api_url = self.config.api_base_url.format(lang=lang)
         # Check cache first
         cache_key_str = (
-            cache_key(lang=self.config.language, **params)
-            if use_cache and self._cache
-            else None
+            cache_key(lang=lang, **params) if use_cache and self._cache else None
         )
         if cache_key_str and self._cache:
             cached_result = await self._cache.get(cache_key_str)
@@ -168,7 +170,8 @@ class WikipediaClient:
             async with self._semaphore:
                 try:
                     async with self._session.get(
-                        self.config.api_url, params=params
+                        api_url,
+                        params=params,
                     ) as response:
 
                         if response.status == 429:
@@ -205,9 +208,47 @@ class WikipediaClient:
                 except aiohttp.ClientError as e:
                     raise NetworkError(f"Network error: {str(e)}") from e
 
+    def _title_to_url(self, title: str) -> str:
+        """Convert a page title to its full Wikipedia URL."""
+        base_url = f"https://{self.config.language}.wikipedia.org/wiki/"
+        return base_url + title.replace(" ", "_")
+
+    def _normalize_title(self, title: str) -> str:
+        # title can be url
+        res = title.strip().replace("_", " ")
+        if title.startswith("http://") or title.startswith("https://"):
+            parts = title.split("/wiki/")
+            if len(parts) == 2:
+                res = parts[1].replace("_", " ")
+                url = f"{self.config.language}.wikipedia.org"
+                if url not in title:
+                    lang = parts[0].split("//")[1].split(".")[0]
+                    if self.config.auto_detect_language:
+                        logger.once(
+                            logging.WARNING,
+                            f"Auto changing language to {lang}. If not desired, than set `auto_detect_language` to False in config",
+                        )
+                        self.config = self.config.with_language(lang)
+                    else:
+                        raise ValidationError(
+                            "title",
+                            f"URL language '{lang}' does not match client language '{self.config.language}'",
+                        )
+            else:
+                raise ValidationError("title", "Invalid Wikipedia URL")
+
+        if not res:
+            raise ValidationError("title", "Title cannot be empty")
+        return unquote_plus(res)
+
     async def search(
-        self, query: str, limit: int = 10, suggestion: bool = False, namespace: int = 0
-    ) -> Union[List[SearchResult], SuggestionResult]:
+        self,
+        query: str,
+        limit: int = 10,
+        suggestion: bool = False,
+        namespace: int = 0,
+        lang: Optional[str] = None,
+    ) -> SearchResults:
         """
         Search for Wikipedia articles.
 
@@ -237,6 +278,8 @@ class WikipediaClient:
             "srnamespace": namespace,
             "srprop": "snippet|titlesnippet|size|wordcount|timestamp",
         }
+        if lang:
+            params["lang"] = lang
 
         if suggestion:
             params["srinfo"] = "suggestion"
@@ -253,24 +296,22 @@ class WikipediaClient:
                     word_count=item.get("wordcount"),
                     size=item.get("size"),
                     timestamp=item.get("timestamp"),
+                    url=self._title_to_url(item["title"]),
                 )
                 for item in search_data
             ]
-
+            suggestion_text = None
             if suggestion:
                 suggestion_text = (
                     response["query"].get("searchinfo", {}).get("suggestion")
                 )
-                return SuggestionResult(
-                    query=query, suggestion=suggestion_text, results=results
-                )
 
-            return results
+            return SearchResults(results=results, suggestion=suggestion_text)
 
         except Exception as e:
             logger.error(f"Search failed for query '{query}': {str(e)}")
             raise
-
+    
     async def get_page(
         self,
         title: Optional[str] = None,
@@ -281,6 +322,9 @@ class WikipediaClient:
         include_links: bool = False,
         include_categories: bool = False,
         include_coordinates: bool = False,
+        include_tables: bool | str = "auto",
+        include_html: bool = False,
+        lang: Optional[str] = None,
     ) -> WikiPage:
         """
         Get a Wikipedia page with optional content.
@@ -294,6 +338,8 @@ class WikipediaClient:
             include_links: Include internal links
             include_categories: Include page categories
             include_coordinates: Include geographic coordinates
+            include_tables: Include extracted tables (True, False, or 'auto')
+            include_html: Include raw HTML content
 
         Returns:
             WikiPage object with requested data
@@ -333,14 +379,16 @@ class WikipediaClient:
             "prop": "|".join(props),
             "inprop": "url",
             "explaintext": "" if include_content else None,
-            "exintro": "" if include_content else None,
         }
 
         if title:
-            params["titles"] = title
+            params["titles"] = self._normalize_title(title)
         else:
             params["pageids"] = page_id
 
+        if lang:
+            params["lang"] = lang
+
         # Remove None values
         params = {k: v for k, v in params.items() if v is not None}
 
@@ -366,13 +414,62 @@ class WikipediaClient:
                 raise DisambiguationError(page_data["title"], options_response)
 
             # Build WikiPage object
+            helper = SectionHelper.from_content(page_data.get("extract", ""))
             wiki_page = WikiPage(
                 title=page_data["title"],
                 page_id=page_data["pageid"],
                 url=page_data["fullurl"],
                 extract=page_data.get("extract"),
                 namespace=page_data.get("ns", 0),
+                summary=helper.first_content(),
+                sections=helper.sections,
+                content=page_data.get("extract", ""),
+                helper=helper,
+                html_content=None,
+                revision_id=None,
+                coordinates=None,
+                last_modified=None,
+                parent_id=None,
             )
+            html = ""
+            should_fetch_html = False
+            if include_html:
+                should_fetch_html = True
+
+            should_fetch_tables = False
+            if include_tables == "auto":
+                for sec in helper.sections:
+                    name = sec.title.lower().strip()
+                    content = sec.content
+                    if not content.strip():
+                        should_fetch_tables = True
+                        break
+                    if (
+                        name == "table"
+                        or "table of" in name
+                        or "list of" in name
+                        or "list:" in name
+                    ):
+                        should_fetch_tables = True
+                        break
+
+            elif include_tables:
+                should_fetch_tables = True
+
+            if should_fetch_tables:
+                should_fetch_html = True
+
+            if should_fetch_html:
+                html = await self.get_page_html(title=title, page_id=page_id, lang=lang)
+                wiki_page.html_content = html
+                if html:
+                    helper = SectionHelper.from_html(html, url=wiki_page.url)
+                    wiki_page.sections = helper.sections
+                    wiki_page.tables = [
+                        s for sec in helper.sections for s in sec.tables
+                    ]
+                    wiki_page.helper = helper
+                    wiki_page.links = helper.links
 
             # Add optional data
             if include_images and "images" in page_data:
@@ -397,6 +494,9 @@ class WikipediaClient:
                     longitude=Decimal(str(coords["lon"])),
                 )
 
+            # with open(f"debug_{wiki_page.title}.html", "w", encoding="utf-8") as f:
+            #     f.write(wiki_page.html_content)
+
             return wiki_page
 
         except (PageNotFoundError, DisambiguationError, RedirectError):
@@ -406,15 +506,20 @@ class WikipediaClient:
             raise WikipediaException(f"Failed to get page: {str(e)}")
 
     async def get_pages_batch(
-        self, titles: List[str], include_content: bool = True
+        self,
+        titles: List[str],
+        include_tables: bool | str = "auto",
+        include_html: bool = False,
+        lang: Optional[str] = None,
     ) -> BatchResult:
         """
         Get multiple pages in batches for efficiency.
 
         Args:
             titles: List of page titles
-            include_content: Include page content
-
+            include_tables: Include extracted tables (True, False, or 'auto')
+            include_html: Include raw HTML content
+            lang: Optional language code for the pages
         Returns:
             BatchResult with successful and failed pages
         """
@@ -426,7 +531,12 @@ class WikipediaClient:
             results = []
             for i in range(0, len(titles), self.config.max_batch_size):
                 chunk = titles[i : i + self.config.max_batch_size]
-                chunk_result = await self._get_pages_chunk(chunk, include_content)
+                chunk_result = await self._get_pages_chunk(
+                    chunk,
+                    include_tables=include_tables,
+                    include_html=include_html,
+                    lang=lang,
+                )
                 results.append(chunk_result)
 
             # Combine results
@@ -440,57 +550,45 @@ class WikipediaClient:
                 successful=successful, failed=failed, total_requested=len(titles)
             )
 
-        return await self._get_pages_chunk(titles, include_content)
+        return await self._get_pages_chunk(
+            titles, include_tables=include_tables, include_html=include_html, lang=lang
+        )
 
     async def _get_pages_chunk(
-        self, titles: List[str], include_content: bool
+        self,
+        titles: List[str],
+        include_tables: bool | str = "auto",
+        include_html: bool = False,
+        lang: Optional[str] = None,
     ) -> BatchResult:
         """Get a chunk of pages (internal method)."""
-        props = ["info", "pageprops"]
-        if include_content:
-            props.append("extracts")
-
-        params = {
-            "prop": "|".join(props),
-            "titles": "|".join(titles),
-            "inprop": "url",
-            "explaintext": "" if include_content else None,
-            "exintro": "" if include_content else None,
-        }
-
-        # Remove None values
-        params = {k: v for k, v in params.items() if v is not None}
 
         try:
-            response = await self._make_request(params)
-            pages = response["query"]["pages"]
-
             successful = []
             failed = []
 
-            for page_data in pages:
-                try:
-                    if "missing" in page_data:
-                        failed.append(
-                            {
-                                "title": page_data.get("title", "unknown"),
-                                "error": "Page not found",
-                            }
-                        )
-                        continue
-
-                    wiki_page = WikiPage(
-                        title=page_data["title"],
-                        page_id=page_data["pageid"],
-                        url=page_data["fullurl"],
-                        extract=page_data.get("extract"),
-                        namespace=page_data.get("ns", 0),
+            res = await asyncio.gather(
+                *[
+                    self.get_page(
+                        title=title,
+                        include_tables=include_tables,
+                        include_html=include_html,
+                        lang=lang,
                     )
-                    successful.append(wiki_page)
-
-                except Exception as e:
+                    for title in titles
+                ],
+                return_exceptions=True,
+            )
+            for title, item in zip(titles, res):
+                if isinstance(item, WikiPage):
+                    successful.append(item)
+                else:
                     failed.append(
-                        {"title": page_data.get("title", "unknown"), "error": str(e)}
+                        {
+                            "title": title,
+                            "error": str(item),
+                            "trace": traceback.format_exc(),
+                        }
                     )
 
             return BatchResult(
@@ -499,13 +597,25 @@ class WikipediaClient:
 
         except Exception as e:
             # All pages failed
-            failed = [{"title": title, "error": str(e)} for title in titles]
+            trace = traceback.format_exc()
+            failed = [
+                {
+                    "title": title,
+                    "error": str(e),
+                    "trace": trace,
+                }
+                for title in titles
+            ]
             return BatchResult(
                 successful=[], failed=failed, total_requested=len(titles)
             )
 
     async def get_summary(
-        self, title: str, sentences: int = 0, characters: int = 0
+        self,
+        title: str,
+        sentences: int = 0,
+        characters: int = 0,
+        lang: Optional[str] = None,
     ) -> PageSummary:
         """
         Get a page summary.
@@ -514,13 +624,14 @@ class WikipediaClient:
             title: Page title
             sentences: Number of sentences (0 = auto)
             characters: Number of characters (0 = auto)
+            lang: Optional language code for the page
 
         Returns:
             PageSummary object
         """
-        params = {
+        params: dict = {
             "prop": "extracts|info",
-            "titles": title,
+            "titles": self._normalize_title(title),
             "inprop": "url",
             "explaintext": "",
             "exintro": "",
@@ -531,6 +642,9 @@ class WikipediaClient:
         elif characters > 0:
             params["exchars"] = characters
 
+        if lang:
+            params["lang"] = lang
+
         response = await self._make_request(params)
         pages = response["query"]["pages"]
 
@@ -546,8 +660,79 @@ class WikipediaClient:
             url=page_data["fullurl"],
         )
 
+    async def get_summary_batch(
+        self,
+        titles: List[str],
+        sentences: int = 0,
+        characters: int = 0,
+        lang: Optional[str] = None,
+    ) -> List[PageSummary]:
+        """
+        Get summaries for multiple pages in batch.
+
+        Args:
+            titles: List of page titles
+            sentences: Number of sentences (0 = auto)
+            characters: Number of characters (0 = auto)
+
+        Returns:
+            List of PageSummary objects
+        """
+        if not titles:
+            return []
+
+        if len(titles) > self.config.max_batch_size:
+            # Process in chunks
+            results = []
+            for i in range(0, len(titles), self.config.max_batch_size):
+                chunk = titles[i : i + self.config.max_batch_size]
+                chunk_result = await self.get_summary_batch(
+                    chunk, sentences, characters
+                )
+                results.extend(chunk_result)
+            return results
+
+        params: dict = {
+            "prop": "extracts|info",
+            "titles": "|".join([self._normalize_title(t) for t in titles]),
+            "inprop": "url",
+            "explaintext": "",
+            "exintro": "",
+        }
+
+        if sentences > 0:
+            params["exsentences"] = sentences
+        elif characters > 0:
+            params["exchars"] = characters
+
+        if lang:
+            params["lang"] = lang
+
+        response = await self._make_request(params)
+        pages = response["query"]["pages"]
+
+        summaries = []
+        for page_data in pages:
+            if "missing" in page_data:
+                continue
+            summaries.append(
+                PageSummary(
+                    title=page_data["title"],
+                    page_id=page_data["pageid"],
+                    extract=page_data.get("extract", ""),
+                    url=page_data["fullurl"],
+                )
+            )
+
+        return summaries
+
     async def geosearch(
-        self, latitude: float, longitude: float, radius: int = 1000, limit: int = 10
+        self,
+        latitude: float,
+        longitude: float,
+        radius: int = 1000,
+        limit: int = 10,
+        lang: Optional[str] = None,
     ) -> List[GeoSearchResult]:
         """
         Search for pages by geographic coordinates.
@@ -581,6 +766,8 @@ class WikipediaClient:
             "gsradius": radius,
             "gslimit": limit,
         }
+        if lang:
+            params["lang"] = lang
 
         response = await self._make_request(params)
         results = response["query"]["geosearch"]
@@ -598,7 +785,12 @@ class WikipediaClient:
             for item in results
         ]
 
-    async def random(self, count: int = 1, namespace: int = 0) -> RandomPageResult:
+    async def random(
+        self,
+        count: int = 1,
+        namespace: int = 0,
+        lang: Optional[str] = None,
+    ) -> RandomPageResult:
         """
         Get random page titles.
 
@@ -613,6 +805,8 @@ class WikipediaClient:
             raise ValidationError("count", "Count must be between 1 and 10")
 
         params = {"list": "random", "rnnamespace": namespace, "rnlimit": count}
+        if lang:
+            params["lang"] = lang
 
         response = await self._make_request(params)
         random_pages = response["query"]["random"]
@@ -621,7 +815,11 @@ class WikipediaClient:
             pages=[page["title"] for page in random_pages], namespace=namespace
         )
 
-    async def suggest(self, query: str) -> Optional[str]:
+    async def suggest(
+        self,
+        query: str,
+        lang: Optional[str] = None,
+    ) -> Optional[str]:
         """
         Get search suggestion for a query.
 
@@ -637,6 +835,8 @@ class WikipediaClient:
             "srinfo": "suggestion",
             "srprop": "",
         }
+        if lang:
+            params["lang"] = lang
 
         response = await self._make_request(params)
         search_info = response["query"].get("searchinfo", {})
@@ -655,7 +855,10 @@ class WikipediaClient:
         response = await self._make_request(params)
         languages = response["query"]["languages"]
 
-        return [Language(code=lang["code"], name=lang["*"]) for lang in languages]
+        return [
+            Language(code=lang["code"], name=lang["*"], english_name=lang.get("en"))
+            for lang in languages
+        ]
 
     async def set_language(self, language: str) -> None:
         """
@@ -702,3 +905,119 @@ class WikipediaClient:
         """Clear the cache."""
         if self._cache:
             await self._cache.clear()
+
+    async def get_page_html_batch(
+        self,
+        titles: List[str],
+        lang: Optional[str] = None,
+    ) -> BatchHTMLResult:
+        """
+        Get HTML content for multiple pages in batch.
+
+        Args:
+            titles: List of page titles
+
+        Returns:
+            list of HTML content strings
+        """
+        # params = {
+        #     "prop": "revisions",
+        #     "rvprop": "content",
+        #     "rvparse": "",
+        #     "rvlimit": 1,
+        #     "titles": "|".join([self._normalize_title(t) for t in titles]),
+        # }
+        # response = await self._make_request(params)
+        # pages = response["query"]["pages"]
+        # for page_data in pages:
+        #     data = ""
+        #     if "revisions" in page_data and page_data["revisions"]:
+        #         rev = page_data["revisions"][0]
+        #         if "slots" in rev and "main" in rev["slots"]:
+        #             rev = rev["slots"]["main"]
+
+        #         if "content" in rev:
+        #             data = rev["content"]
+        #         elif "*" in rev:
+        #             data = rev["*"]
+        #     html_contents.append(data)
+
+        res = await asyncio.gather(
+            *[self.get_page_html(title=title, lang=lang) for title in titles],
+            return_exceptions=True,
+        )
+
+        successful = []
+        failed = []
+        for title, r in zip(titles, res):
+            if isinstance(r, Exception):
+                failed.append({"error": str(r), "title": title})
+            elif isinstance(r, str):
+                successful.append(HTMLResult(title=title, html=r))
+            else:
+                failed.append(
+                    {
+                        "error": f"Unexpected result type: {type(r)}",
+                        "title": title,
+                    }
+                )
+
+        return BatchHTMLResult(
+            successful=successful, failed=failed, total_requested=len(titles)
+        )
+
+    async def get_page_html(
+        self,
+        title: Optional[str] = None,
+        page_id: Optional[int] = None,
+        lang: Optional[str] = None,
+    ) -> str:
+        """
+        Get the HTML content of a Wikipedia page.
+
+        Args:
+            title: Page title (mutually exclusive with page_id)
+            page_id: Page ID (mutually exclusive with title)
+
+        Returns:
+            Full page HTML content
+        """
+        if not title and not page_id:
+            raise ValidationError(
+                "title/page_id", "Either title or page_id must be provided"
+            )
+
+        params = {
+            "prop": "revisions",
+            "rvprop": "content",
+            "rvparse": "",
+            "rvlimit": 1,
+        }
+
+        if title:
+            params["titles"] = self._normalize_title(title)
+        else:
+            params["pageids"] = page_id
+
+        if lang:
+            params["lang"] = lang
+
+        response = await self._make_request(params)
+        pages = response["query"]["pages"]
+
+        if not pages or "missing" in pages[0]:
+            raise PageNotFoundError(title or str(page_id))
+
+        page_data = pages[0]
+        data = ""
+        if "revisions" in page_data and page_data["revisions"]:
+            rev = page_data["revisions"][0]
+            if "slots" in rev and "main" in rev["slots"]:
+                rev = rev["slots"]["main"]
+
+            if "content" in rev:
+                data = rev["content"]
+            elif "*" in rev:
+                data = rev["*"]
+
+        return data
